#include "testApp.h"

#define FRAME_RATE 60
#define BODY_TIME_STAY_MIN 20.0f
#define BODY_TIME_STAY_MAX 30.0f

//--------------------------------------------------------------
void testApp::setup(){

	#ifdef TARGET_OSX
	ofSetDataPathRoot("data/"); // Data can be in Resources folder on Mac
	#endif

	ofEnableSmoothing();
	ofSetFrameRate(FRAME_RATE);
	ofSetRectMode(OF_RECTMODE_CENTER);
//	ofToggleFullscreen();
	ofSetWindowPosition	(0,70);

	box2d.init();
	box2d.setGravity(0, 50);
//	box2d.createFloor();
	box2d.createBounds();
	box2d.checkBounds(true);
	box2d.setFPS(FRAME_RATE);


	ofAddListener(tuio.cursorAdded,this,&testApp::tuioAdded);
	ofAddListener(tuio.cursorRemoved,this,&testApp::tuioRemoved);
	ofAddListener(tuio.cursorUpdated,this,&testApp::tuioUpdated);

	tuio.start(3333);

	timeBefore = ofGetElapsedTimef();
}

//--------------------------------------------------------------
void testApp::update()
{
	float now	= ofGetElapsedTimef();
	float dt	=  now - timeBefore;
	timeBefore = now;

	tuio.getMessage();
	box2d.update();

	for (int i=0;i<bodiesLifeNow.size();i++)
	{
//		printf("ofGetElapsedTimef = %.3f\n", bodiesLife[i]);
		bodiesLifeNow[i] += dt;
		if (bodiesLifeNow[i]>=bodiesLife[i])
			bodiesRemove.push_back(i);		
	}

	for (int i=0;i<bodiesRemove.size();i++)
	{
		printf("REMOVE Body %d\n", i);
		box2d.world->DestroyBody(bodies[i]);
		
		bodiesLifeNow.erase(bodiesLifeNow.begin()+i);
		bodiesLife.erase(bodiesLife.begin()+i);
		shapeCnts.erase(shapeCnts.begin()+i);
		bodies.erase(bodies.begin()+i);
	}

	bodiesRemove.clear();
}

//--------------------------------------------------------------
void testApp::draw(){

	// Background
	ofBackground(255,255,255);

	// Fingers
	for (int i=0;i<fingers.size(); i++)
		fingers[i]->draw();

	// Triangles
//	for (int i=0;i<triangles.size(); i++)
//		triangles[i].draw();

	// Bodies
	ofSetColor(0,0,0);
	ofFill();
	for (int i = 0; i < bodies.size(); i++) 
	{
		b2Body*		body		= bodies[i];
		b2Shape*	s			= body->GetShapeList();
		b2PolygonShape* poly	= (b2PolygonShape*)s;
		int count				= poly->GetVertexCount();
		const b2XForm& xf		= body->GetXForm();
		
		const b2Vec2* localVertices = poly->GetVertices();
		vector<b2Vec2> verts;
		
		for(int j = 0; j < count; j++) {
			verts.push_back(b2Mul(xf, localVertices[j]));
		}

		int shapeCnt = shapeCnts[i];			
		for (int j = 0; j < shapeCnt-1; j++) {
			b2Shape* next = s->GetNext();
			s = next;
			b2PolygonShape* poly = (b2PolygonShape*)next;
			int count = poly->GetVertexCount();
			const b2Vec2* localVertices = poly->GetVertices();
			for(int k = 0; k < count; k++) {
				verts.push_back(b2Mul(xf, localVertices[k]));
			}					
		}


		ofEnableAlphaBlending();
		float life		= bodiesLife[i];
		float lifeNow	= bodiesLifeNow[i];
		float a			= 0.95f;
		float alpha		= (lifeNow <= a*life) ? 1.0f : 1.0f/(life-a*life)*(life-lifeNow) ;
		ofSetColor(0,0,0,alpha*255.0);
		glBegin(GL_TRIANGLES); 
			for(int j = 0; j < verts.size(); j++) {
				glVertex2f(verts[j].x*OFX_BOX2D_SCALE, verts[j].y*OFX_BOX2D_SCALE);
			}
		glEnd();			

		ofDisableAlphaBlending();

	}
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	updateRecord(0,(float)x, (float)y);
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	beginRecord(0);
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	endRecord(0);
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){
}


//--------------------------------------------------------------
void testApp::tuioAdded(ofxTuioCursor & tuioCursor)
{
	beginRecord(tuioCursor.getFingerId());
}

//--------------------------------------------------------------
void testApp::tuioRemoved(ofxTuioCursor & tuioCursor)
{
	endRecord(tuioCursor.getFingerId());
}

//--------------------------------------------------------------
void testApp::tuioUpdated(ofxTuioCursor & tuioCursor)
{
	updateRecord(tuioCursor.getFingerId(), tuioCursor.getX()*ofGetWidth(), tuioCursor.getY()*ofGetHeight());
}


//--------------------------------------------------------------
FingerRecord* testApp::getRecord(int id)
{
	for (int i=0;i<fingers.size();i++)
	{
		if (fingers[i]->id == id)
			return fingers[i];
	}
	return 0;
}

//--------------------------------------------------------------
void testApp::addRecord(int fingerId)
{
	fingers.push_back(new FingerRecord(m_id));
	record.push_back(new FingerId_to_FingerRecord(fingerId, m_id));

	m_id = m_id+1;
}

//--------------------------------------------------------------
void testApp::removeRecord(int id)
{
	for (int i=0;i<fingers.size();i++)
	{
		if (fingers[i]->id == id)
		{
			delete fingers[i];
			fingers.erase(fingers.begin()+i);
			break;
		}
	}
}

//--------------------------------------------------------------
void testApp::beginRecord(int fingerId)
{
	addRecord(fingerId);
}

//--------------------------------------------------------------
void testApp::updateRecord(int fingerId, float x, float y)
{
	FingerId_to_FingerRecord* pRecord=0;
	for (int i=0;i<record.size();i++)
	{
		if (record[i]->fingerId == fingerId)
		{
			pRecord = record[i];
			break;
		}
	}

	if (pRecord)
	{
		if (FingerRecord* pFinger = getRecord(pRecord->fingerRecordId))
			pFinger->update(x,y);
	}

}

//--------------------------------------------------------------
void testApp::endRecord(int fingerId)
{
	FingerId_to_FingerRecord* pRecord=0;
	for (int i=0;i<record.size();i++)
	{
		if (record[i]->fingerId == fingerId)
		{
			pRecord = record[i];
			break;
		}
	}

	if (pRecord)
	{
		if (FingerRecord* pFinger = getRecord(pRecord->fingerRecordId))
		{
			if (pFinger->points.size()>=3)
			{
		
				ofxTriangle tri;
				tri.triangulate(pFinger->points, TRIANGLE_POINTS_MAX);
				triangles.push_back(tri);
				makeComplexBody(tri.triangles);
			}
		
			removeRecord(pRecord->fingerRecordId);
			for (int i=0;i<record.size();i++)
			{
				if (record[i] == pRecord)
				{
					delete pRecord;
					record.erase(record.begin()+i);
					break;
				}
			}
		}
	}

}


//--------------------------------------------------------------
//http://www.psyked.co.uk/box2d/simple-box2d-custom-polygon-creation.htm
void testApp::makeComplexBody(vector<ofxTriangleData>& triangles) {
	int shapeCnt = 0;

	b2BodyDef* bodyDef = new b2BodyDef();
//	bodyDef->position.Set(360/OFX_BOX2D_SCALE, 20/OFX_BOX2D_SCALE);
	bodyDef->linearDamping = 0.25;
	bodyDef->angularDamping = 0.25;
	b2Body* body = box2d.getWorld()->CreateBody(bodyDef);	

	
	for (int i = 0; i < triangles.size(); i++) {
	
		b2PolygonDef* shapeDef = new b2PolygonDef();
		shapeDef->density = 1;
		shapeDef->friction = 5;
		shapeDef->vertexCount = 3;

		shapeDef->vertices[0].Set(triangles[i].a.x/OFX_BOX2D_SCALE, triangles[i].a.y/OFX_BOX2D_SCALE);
		shapeDef->vertices[1].Set(triangles[i].b.x/OFX_BOX2D_SCALE, triangles[i].b.y/OFX_BOX2D_SCALE);
		shapeDef->vertices[2].Set(triangles[i].c.x/OFX_BOX2D_SCALE, triangles[i].c.y/OFX_BOX2D_SCALE);
		
		body->CreateShape(shapeDef);
		shapeCnt++;
	}
	
	shapeCnts.push_back(shapeCnt);
	body->SetMassFromShapes();

	bodies.push_back(body);
	float life = ofRandom(BODY_TIME_STAY_MIN, BODY_TIME_STAY_MAX);
	bodiesLife.push_back(life);
	bodiesLifeNow.push_back(0.0f);
}