#ifndef _TEST_APP
#define _TEST_APP


//--------------------------------------------------------------
/*
	http://www.openframeworks.cc/forum/viewtopic.php?p=18424

	Original code : 
		+ Akira ( http://www.ampontang.com/en )
		
	Remix for "Le Mur Communicant" : 
		+ Julien Gachadoat / v3ga ( http://www.v3ga.net )
	
	Uses :
		+ ofxTriangle by Cyril Diane / kikko ( http://www.kikko.fr/ )
		+ ofxBox2D by Todd Vanderlin ( http://toddvanderlin.com/ )
		+ ofxOsc
		+ ofxTuio (?)
			
*/

//--------------------------------------------------------------
#include "ofMain.h"
#include "ofxTuio.h"
#include "ofxBox2d.h"
#include "ofxTriangle.h"
#include <map>

//--------------------------------------------------------------
#define TRIANGLE_POINTS_MAX 50
//--------------------------------------------------------------
class FingerRecord
{
	public:
		FingerRecord			(int i)
								{
									this->id = i;
								}
		~FingerRecord			()
								{
									printf("DELETE FingerRecord\n");
								}

		void					update	(float x, float y)
										{
											if (points.size() <= TRIANGLE_POINTS_MAX)
												points.push_back( ofPoint(x,y) );
										}

		void					draw	()
										{
											int nbPoints = points.size();
											if (nbPoints>0)
											{
												ofSetColor(200,0,0);
												for (int i=0;i<points.size();i++)
													ofRect(points[i].x,points[i].y,4,4);
												ofSetColor(200,200,200);
												for (int i=0;i<points.size()-1;i++)
													ofLine(points[i].x,points[i].y,points[(i+1)%nbPoints].x,points[(i+1)%nbPoints].y);
											}
										}

		int						id;
		vector<ofPoint>			points;
};

//--------------------------------------------------------------
class FingerId_to_FingerRecord
{
	public:
		FingerId_to_FingerRecord(int fingerId, int fingerRecordId)
		{
			this->fingerId		= fingerId;
			this->fingerRecordId = fingerRecordId;
		}

		~FingerId_to_FingerRecord()
		{}

	int fingerId;
	int fingerRecordId;
};

//--------------------------------------------------------------
class testApp : public ofBaseApp{

	public:
		void setup();
		void update();
		void draw();

		void keyPressed  (int key);
		void keyReleased(int key);
		void mouseMoved(int x, int y );
		void mouseDragged(int x, int y, int button);
		void mousePressed(int x, int y, int button);
		void mouseReleased(int x, int y, int button);
		void windowResized(int w, int h);


		void tuioAdded(ofxTuioCursor & tuioCursor);
		void tuioRemoved(ofxTuioCursor & tuioCursor);
		void tuioUpdated(ofxTuioCursor & tuioCursor);

		// TUIO
		myTuioClient			tuio;

		// Physics
		ofxBox2d				box2d;	
		vector<b2Body*>			bodies;
		vector<int>				shapeCnts;
		vector<float>			bodiesLife;
		vector<float>			bodiesLifeNow;
		vector<int>				bodiesRemove;
		void					makeComplexBody	(vector <ofxTriangleData>& triangles);

		// Fingers
		vector<FingerRecord*>	fingers;
		void					addRecord		(int fingerId);
		void					removeRecord	(int id);
		FingerRecord*			getRecord		(int id);

		void					beginRecord		(int fingerId);
		void					updateRecord	(int fingerId, float x, float y);
		void					endRecord		(int fingerId);
		
		int						m_id;


		vector<FingerId_to_FingerRecord*>	record;
		
		// Triangles
		vector<ofxTriangle>		triangles;		

		// Timing
		float					timeBefore;

};

#endif
